% advanced signal processing seminar 1
% daniel walch
% erwin nindl

clear all;
clc;

% include library
addpath('SumProductLab_R3.00')

disp('Sum-Product (Belief Propagation) Algorithm: An Example')
disp('H.A. Loeliger, “An Introduction to Factor Graphs”,')
disp('  IEEE Signal Processing Mag., Jan. 2004.')
disp('------------------------------------------------------')
disp(' ')


% no loopy factor graph
global loopy
loopy = 1;

% instantiate nodes

e1 = evident_node(1);
e2 = evident_node(2);
e3 = evident_node(3);
e4 = evident_node(4);

d1 = equ_node(7);
d2 = equ_node(8);
d3 = equ_node(9);
d4 = equ_node(10);

eq = equ_node(11);
c1 = even_parity_node(12);

% define connection
e1.setup_link({d1});
e2.setup_link({d2});
e3.setup_link({d3});
e4.setup_link({d4});

d1.setup_link({e1 c1});
d2.setup_link({e2 c1});
d3.setup_link({e3 eq});
d4.setup_link({e4 eq});

c1.setup_link({d1 d2 eq});
eq.setup_link({d3 d4 c1});

e1.setup_init_msg([0.9 0.1]);
e2.setup_init_msg([0.9 0.1]);
e3.setup_init_msg([0.1 0.9]);
e4.setup_init_msg([0.9 0.1]);

x_estimate = zeros(2,4);

code = zeros(1,4);
code(1) = (e1.outbound_msg{1}(2) > 0.5);
code(2) = (e2.outbound_msg{1}(2) > 0.5);
code(3) = (e3.outbound_msg{1}(2) > 0.5);
code(4) = (e4.outbound_msg{1}(2) > 0.5);


disp('Initial Message:')
disp(code)

disp('Output of the factor graph')
disp('------------------------------------------')

figure;
hold on;
title('Iterative bit-wise MAP decoding'),
xlabel('i'), ylabel('p_X_i_|_Y(x_i|y)'),
hfig_old = bar(1:size(x_estimate,2), x_estimate', 'stacked');
colormap(summer)
hold off
legend(hfig_old, 'p(x_i=0|Y)','p(x_i=1|Y)','Location','NorthEastOutside');
hold on
%set(gca,'xlim',[-0.5 size(x_estimate,2)+0.5]);
set(gca,'ylim',[0 1]);


% iteration
for i = 1:20
    
    % update network
    d1.update_node();
    d2.update_node();
    d3.update_node();
    d4.update_node();
    c1.update_node();
    eq.update_node();
    
    % result
    p = marginal(e1,d1);
    code(1) = p(2) > 0.5;
    x_estimate(:,1) = p;
    p = marginal(e2,d2);
    code(2) = p(2) > 0.5;
    x_estimate(:,2) = p;
    p = marginal(e3,d3);
    code(3) = p(2) > 0.5;
    x_estimate(:,3) = p;
    p = marginal(e4,d4);
    code(4) = p(2) > 0.5;
    x_estimate(:,4) = p;

    % text output
    fprintf('t: %i\n',i);
    fprintf(' p(x_i=0|y):     %.2f   %.2f   %.2f   %.2f\n', x_estimate(1,:));
    fprintf(' p(x_i=1|y):     %.2f   %.2f   %.2f   %.2f\n', x_estimate(2,:));
    disp('  ');
    
    hfig = bar(1:size(x_estimate,2), x_estimate', 'stacked');
    legend(hfig, 'p(x_i=0|Y)','p(x_i=1|Y)','Location','NorthEastOutside');
    title(sprintf('Iterative bit-wise MAP decoding, t: %i', i)),
    delete(hfig_old);
    hfig_old = hfig;
    drawnow;
    pause;
    
end



%eof